@@ -29,45 +29,50 @@ class OAuth2Auth(Connection):
             Any class instance will be accepted as long as it has "read_token" and "write_token" methods.
         :type token_storage: :class:`xmatters.utils.TokenFileStorage`, optional
         """
+        self.base_url = base_url
         self.client_id = client_id
         self.token_storage = token_storage
         self.username = username
         self.password = password
         self._token = token
         self.session = None
-        token_url = '{}{}'.format(base_url, self._endpoints.get('token'))
+        token_url = '{}{}'.format(self.base_url, self._endpoints.get('token'))
         client = LegacyApplicationClient(client_id=self.client_id)
         auto_refresh_kwargs = {'client_id': self.client_id}
         token_updater = self.token_storage.write_token if self.token_storage else None
         self.session = OAuth2Session(client=client, auto_refresh_url=token_url,
                                      auto_refresh_kwargs=auto_refresh_kwargs,
                                      token_updater=token_updater)
-        self.session.token = self.get_token()
-
-        # update storage token if differs from self.token
-        if self.token_storage and self.token_storage.read_token() != self.token:
-            self.token_storage.write_token(self.token)
-
-        super(OAuth2Auth, self).__init__(base_url, self.session, **kwargs)
+        self._set_token()
+        self._update_storage()
+        super(OAuth2Auth, self).__init__(self, **kwargs)
 
     def refresh_token(self):
-        return self.session.refresh_token(token_url=self.session.auto_refresh_url, refresh_token=self._token,
-                                          timeout=3,
-                                          kwargs=self.session.auto_refresh_kwargs)
+        # session token automatically set
+        self.session.refresh_token(token_url=self.session.auto_refresh_url, refresh_token=self._token,
+                                   timeout=3, kwargs=self.session.auto_refresh_kwargs)
+        self._update_storage()
 
     def fetch_token(self):
-        return self.session.fetch_token(token_url=self.session.auto_refresh_url, username=self.username,
-                                        password=self.password, include_client_id=True, timeout=3)
+        # session token automatically set
+        self.session.fetch_token(token_url=self.session.auto_refresh_url, username=self.username,
+                                 password=self.password, include_client_id=True, timeout=3)
+        self._update_storage()
 
-    def get_token(self):
+    def _update_storage(self):
+        if self.token_storage:
+            self.token_storage.write_token(self.token)
+
+    def _set_token(self):
         if self._token and isinstance(self._token, dict):
-            return self._token
+            self.token = self._token
+            self._update_storage()
         elif self._token and isinstance(self._token, str):
-            return self.refresh_token()
+            self.refresh_token()
         elif None not in (self.username, self.password):
-            return self.fetch_token()
+            self.fetch_token()
         elif self.token_storage and self.token_storage.read_token():
-            return self.token_storage.read_token()
+            self.token = self.token_storage.read_token()
         else:
             raise err.XMSessionError('Unable to obtain token with provided arguments')
 
@@ -104,8 +109,9 @@ class BasicAuth(Connection):
         self.username = username
         self.password = password
         self.session = requests.Session()
+        self.base_url = base_url
         self.session.auth = (self.username, self.password)
-        super(BasicAuth, self).__init__(base_url, self.session, **kwargs)
+        super(BasicAuth, self).__init__(self, **kwargs)
 
     def __repr__(self):
         return '<{}>'.format(self.__class__.__name__)
