@@ -1,3 +1,6 @@
+# TODO: process boolean params
+# TODO: upper 'status' params
+
 import xmatters.factories as factory
 import xmatters.xm_objects.forms
 from xmatters.connection import ApiBridge
@@ -7,12 +10,12 @@ from xmatters.xm_objects.device_types import DeviceTypes
 from xmatters.xm_objects.dynamic_teams import DynamicTeam
 from xmatters.xm_objects.event_supressions import EventSuppression
 from xmatters.xm_objects.events import Event
-from xmatters.xm_objects.groups import Group
+from xmatters.xm_objects.groups import Group, GroupQuota
 from xmatters.xm_objects.import_jobs import Import
 from xmatters.xm_objects.incidents import Incident
 from xmatters.xm_objects.oncall import OnCall
 from xmatters.xm_objects.oncall_summary import OnCallSummary
-from xmatters.xm_objects.people import Person
+from xmatters.xm_objects.people import Person, UserQuota
 from xmatters.xm_objects.plans import Plan
 from xmatters.xm_objects.roles import Role
 from xmatters.xm_objects.scenarios import Scenario
@@ -135,7 +138,7 @@ class DeviceNamesEndpoint(ApiBridge):
     # TODO: Test params
     def get_device_names(self, device_types=None, search=None, sort_by=None, sort_order=None,
                          at_time=None, offset=None, limit=None):
-        params = {'search': ' '.join(search) if search else None,
+        params = {'search': self.process_search_param(search),
                   'sortBy': sort_by,
                   'sortOrder': sort_order,
                   'deviceTypes': device_types,
@@ -193,7 +196,7 @@ class DynamicTeamsEndpoint(ApiBridge):
     # TODO: Test params
     def get_dynamic_teams(self, search=None, operand=None, fields=None, supervisors=None, sort_by=None,
                           sort_order=None, offset=None, limit=None):
-        params = {'search': ' '.join(search) if search else None,
+        params = {'search': self.process_search_param(search),
                   'operand': operand,
                   'fields': fields,
                   'supervisors': supervisors,
@@ -263,7 +266,7 @@ class EventsEndpoint(ApiBridge):
                   'sortBy': sort_by,
                   'sortOrder': sort_order,
                   'submitterid': submitter_id,
-                  'search': ' '.join(search) if search else None,
+                  'search': self.process_search_param(search),
                   'targetedRecipients': targeted_recipients,
                   'resolvedUsers': resolved_users,
                   'from': self.process_time_param(from_time),
@@ -389,7 +392,7 @@ class FormsEndpoint(ApiBridge):
     # TODO: Test params
     def get_forms(self, search=None, operand=None, fields=None, enabled_for=None, plan_type=None, sort_by=None,
                   sort_order=None, trigger_type=None, offset=None, limit=None):
-        params = {'search': ' '.join(search) if search else None,
+        params = {'search': self.process_search_param(search),
                   'operand': operand,
                   'fields': fields,
                   'enabledFor': enabled_for,
@@ -419,7 +422,8 @@ class FormsEndpoint(ApiBridge):
 
 class GroupsEndpoint(ApiBridge):
     _endpoints = {'get_groups': '/groups',
-                  'get_group_by_id': '/groups/{group_id}'}
+                  'get_group_by_id': '/groups/{group_id}',
+                  'get_license_quotas': '/groups/license-quotas'}
 
     def __init__(self, parent):
         super(GroupsEndpoint, self).__init__(parent)
@@ -427,7 +431,7 @@ class GroupsEndpoint(ApiBridge):
     # TODO: Test params
     def get_groups(self, search=None, operand=None, fields=None, sites=None, members=None, members_exists=None,
                    sort_by=None, sort_order=None, status=None, supervisors=None, offset=None, limit=None):
-        params = {'search': ' '.join(search) if search else None,
+        params = {'search': self.process_search_param(search),
                   'operand': operand,
                   'fields': fields,
                   'sites': sites,
@@ -450,6 +454,11 @@ class GroupsEndpoint(ApiBridge):
         data = self.con.get(url, params=params)
         return Group(self, data) if data else None
 
+    def get_license_quotas(self):
+        url = self.build_url(self._endpoints.get('get_license_quotas'))
+        data = self.con.get(url)
+        return GroupQuota(data) if data else None
+
     # TODO: Test
     def create_group(self, data):
         url = self.build_url(self._endpoints.get('get_groups'))
@@ -515,7 +524,7 @@ class IncidentsEndpoint(ApiBridge):
     def get_incidents(self, request_id=None, search=None, operand=None, fields=None, sites=None, status=None,
                       severity=None, offset=None, limit=None):
         params = {'requestId': request_id,
-                  'search': ' '.join(search) if search else None,
+                  'search': self.process_search_param(search),
                   'operand': operand,
                   'fields': fields,
                   'sites': sites,
@@ -606,7 +615,8 @@ class OnCallSummaryEndpoint(ApiBridge):
 
 class PeopleEndpoint(ApiBridge):
     _endpoints = {'get_people': '/people',
-                  'get_person_by_id': '/people/{person_id}'}
+                  'get_person_by_id': '/people/{person_id}',
+                  'get_license_quotas': '/people/license-quotas'}
 
     def __init__(self, parent):
         super(PeopleEndpoint, self).__init__(parent)
@@ -617,7 +627,7 @@ class PeopleEndpoint(ApiBridge):
                    groups=None, groups_exist=None, roles=None, supervisors=None, created_from_time=None,
                    created_to_time=None, created_before_time=None, created_after_time=None, sort_by=None,
                    sort_order=None, at_time=None, limit=None):
-        params = {'search': ' '.join(search) if search else None,
+        params = {'search': self.process_search_param(search),
                   'operand': operand,
                   'fields': fields,
                   'propertyNames': property_names,
@@ -664,6 +674,11 @@ class PeopleEndpoint(ApiBridge):
         data = self.con.get(url, params=params)
         return Pagination(self, data, Person, limit=limit) if data.get('data') else []
 
+    def get_license_quotas(self):
+        url = self.build_url(self._endpoints.get('get_license_quotas'))
+        data = self.con.get(url)
+        return UserQuota(data) if data else None
+
     def create_person(self, data):
         url = self.build_url(self._endpoints.get('get_people'))
         data = self.con.post(url, data=data)
@@ -699,7 +714,7 @@ class PlansEndpoint(ApiBridge):
         params = {'planType': plan_type,
                   'enabled': enabled,
                   'enabledFor': enabled_for,
-                  'search': ' '.join(search) if search else None,
+                  'search': self.process_search_param(search),
                   'operand': operand,
                   'fields': fields,
                   'sortBy': sort_by,
@@ -775,7 +790,7 @@ class ScenariosEndpoint(ApiBridge):
 
     # TODO: Test params
     def get_scenarios(self, search=None, operand=None, enabled_for=None, offset=None, limit=None):
-        params = {'search': ' '.join(search) if search else None,
+        params = {'search': self.process_search_param(search),
                   'operand': operand,
                   'enabledFor': enabled_for,
                   'offset': offset,
@@ -805,7 +820,7 @@ class ServicesEndpoint(ApiBridge):
 
     # TODO: Test params
     def get_services(self, search=None, operand=None, fields=None, owned_by=None, offset=None, limit=None):
-        params = {'search': ' '.join(search) if search else None,
+        params = {'search': self.process_search_param(search),
                   'operand': operand,
                   'fields': fields,
                   'ownedBy': owned_by,
@@ -855,7 +870,7 @@ class SitesEndpoint(ApiBridge):
     # TODO: Test params
     def get_sites(self, search=None, operand=None, fields=None, sort_by=None, sort_order=None, country=None,
                   geocoded=None, status=None, offset=None, limit=None):
-        params = {'search': ' '.join(search) if search else None,
+        params = {'search': self.process_search_param(search),
                   'operand': operand,
                   'fields': fields,
                   'sortBy': sort_by,
